var gulp = require('gulp'),
	clean = require('gulp-clean'),
	rename = require('gulp-rename'),
	sass = require('gulp-sass')(require('sass')),
	autoprefixer = require('gulp-autoprefixer'),
	cssnano = require('gulp-cssnano'),
	webpack = require('webpack'),
	webpackStream = require('webpack-stream');

gulp.task('clean-dist', function () {
	return gulp.src('./dist/*')
		.pipe(clean());
});

gulp.task('sass-build', function () {
	return gulp.src('./src/scss/**/*.scss')
		.pipe(sass({outputStyle: 'expanded'}))
		.pipe(autoprefixer(['last 5 versions', '> 1%'], {cascade: true}))
		.pipe(gulp.dest('./src/css'))
		.pipe(gulp.dest('./dist/css'))
		.pipe(cssnano({
			zindex: false
		}))
		.pipe(rename({
			suffix: '.min'
		}))
		.pipe(gulp.dest('./dist/css'))
});

gulp.task('js-build', function () {
	return gulp.src('./src/js/hs-navbar-vertical-aside.js')
		.pipe(webpackStream({
			mode: 'development',
			devtool: 'source-map',
			output: {
				library: 'HSSideNav',
				libraryTarget: 'umd',
				libraryExport: 'default',
				filename: 'hs-navbar-vertical-aside.min.js',
			},
			optimization: {
				minimize: true
			},
			module: {
				rules: [
					{
						test: /\.(js)$/,
						exclude: /(node_modules)/,
						loader: 'babel-loader',
						options: {
							presets: [
								["@babel/preset-env"]
							]
						}
					}
				]
			},
			externals: {
				jquery: 'jQuery'
			}
		}))
		.pipe(gulp.dest('./dist/js'));
});

gulp.task('main-watch', function () {
	gulp.watch('./src/scss/**/*.scss', gulp.series('sass-build'));
	gulp.watch('./src/js/hs-navbar-vertical-aside.js', gulp.series('js-build'));
});

// Default Task
gulp.task('default', gulp.series('sass-build', 'js-build'));